# frozen_string_literal: true

require_relative 'core/util/util'

module Zashoku
  Version = [1, 7, 1].freeze

  Root    = File.expand_path('../', __dir__)

  DefConf = {
    app: {
      root: Root,
      name: 'zashoku',
      commands: {},
      modules: {
        viewer: [],
        daemon: [],
      },
      log_level: {
        viewer: :warn,
        daemon: :info
      },
      save_config: false,
      save_history: false,
      persistent_daemon: false,
      net: {
        host: 'localhost',
        port: 26_119
      },
      msg: {
        quit: 'Type :quit<enter> to quit'
      }
    },
    core: {
      commands: {
        'quit'          => :cleanup,
        'q'             => :cleanup,
        'enter_command' => :get_user_command,
        'change_view'   => :change_view,
        'map_key'       => :map_key,
        'search'        => :search,
        'zv'            => :print_version
      },
      net: {
        bsize: 4096
      }
    }
  }.freeze

  CConf = const_defined?(:AppConf) ? Zashoku::Util.deep_merge(DefConf, AppConf) : DefConf

  LoadPaths = [
    File.join(Zashoku::CConf[:app][:root], 'modules')
  ].freeze

  EOF = 'ZASHOKU_EOF'
end
