# frozen_string_literal: true

require 'yaml'
require 'fileutils'

module Zashoku
  class Config < Controller
    CONF_DIR  = File.join(Dir.home, '.config/', Zashoku::CConf[:app][:name])
    CONF_FILE = File.join(CONF_DIR, '/conf.yml')

    def initialize
      FileUtils.mkdir_p(CONF_DIR) if save? && !File.directory?(CONF_DIR)
      reload
      save unless File.exist?(self.class::CONF_FILE)
    end

    def save?
      Zashoku::CConf[:app][:save_config]
    end

    def reload
      @conf = Zashoku::Util.deep_merge(default_conf, user_conf)
      save
    end

    def base_conf
      {}
    end

    def default_conf
      module_conf.reduce(base_conf) do |mem, mc|
        Zashoku::Util.deep_merge(mem, mc)
      end
    end

    def module_conf
      {}
    end

    def user_conf
      Util.get_yaml(self.class::CONF_FILE)
    end

    def pref_model(title, value, trail)
      Item.new('title' => title, 'value' => value, 'trail' => trail)
    end

    def items(c = @conf, pre = '', path = [])
      return pref_model(pre, c, path) unless c.respond_to?(:map)
      c.map do |title, pref|
        # next if IGNORE.key? title
        t = "#{pre}_#{title}"
        pth = path + [title]
        case pref
        when Hash
          pref.map do |subt, subpref|
            items(subpref, "#{pre}_#{title}_#{subt}", [title, subt])
            #pref_model("#{title}_#{subt}", subpref, [title, subt])
          end
        when Array
          pref_model(t, pref.join(':'), pth)
        else
          pref_model(t, pref, pth)
        end
      end.flatten #.compact
    end

    def parse_dir(path)
      {
        '$conf_dir'   => CONF_DIR,
        '~' => Dir.home
      }.inject(path) { |p, r| p.gsub(r[0], r[1]) }
    end

    def set(trail, destination)
      trail = [trail] if trail.class == String

      trail[0...-1].inject(@conf, :fetch)[trail.last] = destination
      save
      changed
      notify_observers
    end

    def get(trail)
      @conf.dig(*trail)
    end

    def save
      return unless save?
      File.open(self.class::CONF_FILE, 'w') { |f| f.write(YAML.dump(@conf)) }
    end
  end
end
