# frozen_string_literal: true

require 'yaml'

module Zashoku
  module Module
    def self.require_module(mod)
      mod = mod.downcase
      Zashoku::LoadPaths.map do |path|
        mod_path = File.join(path, "#{mod}/lib/#{mod}.rb")
        next unless File.exist?(mod_path)
        begin
          require mod_path
        rescue LoadError => e
          Zashoku.logger.error("error: #{e} while loading #{mod}")
          false
        end
      end.any?
    end

    def self.load(modules)
      require_all(modules).map do |mod|
        [mod.name.split(':').last.downcase, mod]
      end.to_h
    end

    def self.require_all(modules)
      modules.map do |mod|
        if require_module(mod)
          Zashoku.logger.info("loaded module #{mod}")
          Zashoku.const_get(mod)
        else
          Zashoku.logger.error("failed to load module #{mod}")
          nil
        end
      end.compact
    end

    ###

    def view; end

    def controller; end

    def view_config
      Util.get_yaml(File.join(self::Root, 'config/view.yml'))
    end

    def daemon_config
      Util.get_yaml(File.join(self::Root, 'config/daemon.yml'))
    end
  end
end
