# frozen_string_literal: true

require_relative 'widget'

module Zashoku
  module Statusline
    def self.new
      Zashoku::Statusline::Statusline.new
    end

    class Statusline
      include Observable

      def initialize
        @widgets = Zashoku.modules.keys.zip(Zashoku.modules.map do |_k, m|
          m.widgets if m.respond_to?(:widgets)
        end).to_h.compact
        Zashoku.logger.info("loaded widgets for #{@widgets.keys.join(', ')}")
        refresh
      end

      def change_screen_size
        refresh
        changed!
      end

      def changed!
        changed
        notify_observers
      end

      def event(event)
        case event['event']
        when 'refresh_widget'
          @widgets[event['sender']][event['widget']].refresh
          refresh
          changed!
        end
      end

      def refresh
        @cl = Zashoku::conf.get(%w[color main])
        @attrs = @widgets.map do |_k, m|
          m.map { |j, w| w.attr }
        end.flatten.reduce(&:merge)

        format = Zashoku::conf.get(%w[format statusline])

        @items_formatted = Zashoku::Formatter.format_line(format, @attrs)
      end

      def draw
        print "\e[#{Util::Term.rows - 1};1H\e[0m#{@cl}#{@items_formatted}\e[K"
      end
    end
  end
end
