# frozen_string_literal: true

module Zashoku
  module Util
    module Term
      {
        save: `tput smcup`,
        restore: `tput rmcup`,
        hide_cursor: `tput civis`,
        show_cursor: `tput cnorm`,
        clear_line: "\e[2K",
        bold: `tput bold`,
        nobold: `tput sgr0`,
        clear: "\033[2J"
      }.each do |command, code|
        define_singleton_method(command) do
          print code
          self
        end
      end

      def self.echo_off
        `stty -echo`
        self
      end

      def self.echo_on
        `stty echo`
        self
      end

      def self.cols
        HighLine::SystemExtensions.terminal_size[0]
      end

      def self.rows
        HighLine::SystemExtensions.terminal_size[1]
      end

      def get_key
        STDIN.getch
             .gsub("\r", 'enter')
             .gsub(' ',  'space')
             .gsub('A',  'up')
             .gsub('B',  'down')
             .gsub('C',  'right')
             .gsub('D',  'left')
             .gsub("\e", 'skip')
             .gsub('[',  'skip')
      end

      def self.reset
        restore.show_cursor.echo_on
      end

      def self.ini
        save.hide_cursor.echo_off
      end
    end
  end
end
