# frozen_string_literal: true

require 'base64'

module Zashoku
  module Util
    def self.encode_object(d)
      Base64.strict_encode64(Marshal.dump(d))
    end

    def self.decode_object(e)
      return e if e.class == Hash
      Marshal.load(Base64.decode64(e))
    rescue ArgumentError
      {}
    rescue TypeError
      {}
    end

    def self.deep_merge(this, that)
      this&.merge(that || {}) do |_k, old, nu|
        case old
        when Hash
          deep_merge(old, nu)
        #when Array
        #  old + nu
        else
          nu
        end
      end
    end

    def self.get_yaml(file, nil_value = {})
      YAML.safe_load(File.open(file, 'r', &:read)) || nil_value
    rescue Errno::ENOENT
      nil_value
    end

    def self.put_yaml(file, data)
      File.open(file, 'w') { |f| f.puts YAML.dump(data) }
    end

    def self.alert(msg)
      print "\e[#{Term.rows};0H\e[31m#{msg}\e[K"
    end
  end
end
