# frozen_string_literal: true

require 'fileutils'
require 'pathname'

module Zashoku
  module Generator
    TemplateDir = File.join(Zashoku::Root, 'templates')
    Extensions = 'rb,yml,md,gemspec'

    def self.generate(template, name)
      raise 'error, no name given' unless name
      generator =
        case template
        when 'module'
          require_relative 'generator/module_generator'
          ModuleGenerator.new(name)
        when 'app'
          require_relative 'generator/app_generator'
          AppGenerator.new(name)
        else
          raise "Template #{template} not found"
        end

      generator.generate!
      puts "generated #{generator.generated.join(', ')}"
    end

    class Generator
      attr_accessor :generated

      def initialize(name)
        @generated = []
        @name = name
        @name_path = Pathname.new(File.absolute_path(@name))
      end

      def pairs
        @pairs || @pairs = {
          /<name>/ => @name,
          /<Name>/ => @name.capitalize,
          /<zv_short>/ => Zashoku::Version.first(2).join('.'),
          /<zv_long>/ => Zashoku::Version.join('.')
        }
      end

      def refactor(thing)
        pairs.reduce(thing.to_s) { |f, t| f = f.gsub(t[0], t[1]) }
      end

      def transfer(template, output)
        input_dir  = Pathname.new(File.join(TemplateDir, template))
        output_dir = Pathname.new(output)
        raise "path #{output_dir} already exists" if Dir.exist?(output_dir)
        Dir.glob("#{input_dir}/**/*").each do |file|
          next if Dir.exist?(file)

          new_file =
            File.join(
              output_dir,
              refactor(Pathname.new(file).relative_path_from(input_dir))
            )

          FileUtils.mkdir_p(File.dirname(new_file))

          File.open(new_file, 'w') do |f|
            f.print refactor(File.open(file, 'r', &:read))
          end

          FileUtils.chmod(File.stat(file).mode, new_file)
        end

        output
      end

      def generate!; end
    end
  end
end
