# frozen_string_literal: true

require_relative 'core/options'
require_relative 'constants'

module Zashoku
  autoload :Generator, File.join(Root, 'lib/generator')
  autoload :Viewer,    File.join(Root, 'lib/viewer')
  autoload :Daemon,    File.join(Root, 'lib/daemon')

  class << self
    attr_accessor :conf, :modules, :logger
  end

  def self.command(message)
    message.keys.map(&:to_sym).zip(message.values).to_h

    modules[message[:mod]].send(message[:meth], *message[:args])
  end


  def self.main(args)
    @options = Options.parse(args)
    host = @options[:host].key?(:host) ? @options[:host][:host] : Zashoku::CConf[:app][:net][:host]
    port = @options[:host].key?(:port) ? @options[:host][:port] : Zashoku::CConf[:app][:net][:port]

    Zashoku.modules = []

    if @options[:generate]
      Generator.generate(@options[:template], @options[:generate_name])
    elsif @options[:server_command]
      print command_server(@options[:server_command], host, port)[:payload]
    else
      unless command_server('up?', host, port)[:status]
        @options[:daemon] = true
        command_server('start', host, port)
        sleep(0.2) until command_server('up?', host, port)[:status]
      end
      Viewer.new(host: host, port: port)
    end
  end

  def self.start_daemon(port)
    if @options[:daemon]
      fork {
        Process.daemon()
        d = Daemon.new(port: port)
        d.listen
      }
    else
      d = Daemon.new(port: port)
      d.listen
    end
  end

  def self.command_server(command, host, port)
    case command
    when 'start'
      start_daemon(port)
    when 'restart'
      command_server('stop')
      start_daemon
    else
      require_relative 'core/net/client'
      Net::Client.command(host, port, command)
    end
  end
end
